##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################
import sys
from string import join
import Shared.DC.ZRDB.Connection
from App.special_dtml import HTMLFile
from ExtensionClass import Base
import Acquisition

def field_to_string(d):
    type_name = d['TYPE_NAME']
    if type_name == 'SHORT':
        s = 'SMALLINT'
    elif type_name == 'LONG':
        s = 'INTEGER'
    elif type_name == 'TEXT':
        s = 'CHAR(' + str(d['CHARACTER_LENGTH']) + ')'
    elif type_name == 'VARYING':
        s = 'VARCHAR(' + str(d['CHARACTER_LENGTH']) + ')'
    elif type_name == 'INT64':
        if d['FIELD_SUB_TYPE'] == 1:
            s = 'NUMERIC'
        else:
            s = 'DECIMAL'
        s += '(' + str(d['FIELD_PRECISION']) + ',' + str(d['FIELD_SCALE'] * -1) + ')'
    elif type_name == 'BLOB':
        s = 'BLOB SUB_TYPE ' + str(d['FIELD_SUB_TYPE'])
    else:
        s = type_name

    if d.get('NULL_FLAG'):
        s += ' NOT NULL '

    if d.get('DEFAULT_SOURCE'):
        s += d['DEFAULT_SOURCE']

    return s

class Connection(Shared.DC.ZRDB.Connection.Connection):
    _isAnSQLConnection=1

    manage_options=Shared.DC.ZRDB.Connection.Connection.manage_options+(
        {'label': 'Browse', 'action':'manage_browse'},
        )

    manage_tables=HTMLFile('dtml/tables',globals())
    manage_browse=HTMLFile('dtml/browse',globals())

    def tpValues(self):
        r=[]
        if not hasattr(self, '_v_database_connection'):
            return r
        c=self._v_database_connection

        for d in c.tables(rdb=0):
            r.append(TableBrowser(d, c))
        for d in c.procedures():
            r.append(ProcedureBrowser(d))
        for d in c.generators():
            r.append(GeneratorBrowser(d))

        return r

    def __getitem__(self, name):
        if name=='tableNamed':
            if not hasattr(self, '_v_tables'): self.tpValues()
            return self._v_tables.__of__(self)
        raise KeyError, name

class Browser(Base):
    info=None
    icon=''
    Description=''
    def __init__(self, d, c=None):
        self.__dict__.update(d)
        self._c = c

    def tpValues(self):
        return []

    def tpId(self): return ''
    def Name(self): return self.tpId()
    def Type(self): return ''
    def Params(self): return ''
    def Source(self): return ''

class TableBrowser(Browser, Acquisition.Implicit):
    info=HTMLFile('dtml/table_info',globals())

    def tpValues(self):
        r=[]

        for d in self._c.columns(self.TABLE_NAME):
            r.append(ColumnBrowser(d))

        consts = self._c.constraints(self.TABLE_NAME)
        for d in consts:
            r.append(ConstraintBrowser(consts[d]))

        check_consts = self._c.check_constraints(self.TABLE_NAME)
        for d in check_consts:
            r.append(CheckConstraintBrowser(check_consts[d]))

        for d in self._c.triggers(self.TABLE_NAME):
            r.append(TriggerBrowser(d))

        r.append(SourceBrowser(self.VIEW_SOURCE))

        return r

    def tpId(self): return self.TABLE_NAME
    def icon(self):
        if self.VIEW_SOURCE: return 'view'+'.gif'
        elif self.SYSTEM_FLAG: return 'stable'+'.gif'
        else: return 'table'+'.gif'
    def Type(self):
        if self.VIEW_SOURCE: return 'VIEW'
        elif self.SYSTEM_FLAG: return 'SYSTEM_TABLE'
        else: return 'TABLE'

class ProcedureBrowser(Browser, Acquisition.Implicit):
    icon='procedure'+'.gif'

    def tpValues(self):
        param = ''
        inp = [e['NAME'] + ' ' + field_to_string(e) for e in self.IN_PARAMS]
        outp = [e['NAME'] + ' ' + field_to_string(e) for e in self.OUT_PARAMS]
        if inp:
            param += '(' + join(inp, ',') + ')'
        if outp:
            param += ' RETURNS(' + join(outp, ',') + ')'

        return [SourceBrowser(self.PROCEDURE_SOURCE, param)]

    def tpId(self): return self.PROCEDURE_NAME
    def Type(self): return 'PROCEDURE'

class GeneratorBrowser(Browser, Acquisition.Implicit):
    def tpValues(self):
        return []
    def tpId(self): return self.GENERATOR_NAME
    def Type(self):
        if self.SYSTEM_FLAG == 0:
            return 'SEQUENCE'
        else:
            return 'GENERATOR'
    def icon(self):
        if self.SYSTEM_FLAG == 0:
            return 'sequence'+'.gif'
        else:
            return 'generator'+'.gif'
    def Description(self):
        return '#' + str(self.GENERATOR_COUNT)

class ColumnBrowser(Browser):
    field_icons={ 'SHORT': 'int', 'LONG': 'int', 'QUAD': 'int',
        'FLOAT': 'float', 'DATE': 'date', 'TIME': 'time', 'TEXT': 'text',
        'INT64': 'int', 'DOUBLE': 'float', 'TIMESTAMP': 'datetime',
        'VARYING': 'text', 'CSTRING': 'text', 'BLOB_ID': 'int', 'BLOB': 'bin',}
    def tpId(self): return self.NAME
    def Type(self): return self.TYPE_NAME
    def icon(self): return self.field_icons.get(self.TYPE_NAME, 'field')+'.gif'
    def Description(self):
        return field_to_string(self.__dict__)

class ConstraintBrowser(Browser):
    const_icons={'PRIMARY KEY':'key_primary', 
                'INDEX':'index', 
                'UNIQUE':'key', 
                'FOREIGN KEY':'key_foreign'}
    def tpValues(self):
        return []
    def tpId(self): return self.INDEX_NAME
    def icon(self): return self.const_icons[self.CONST_TYPE]+'.gif'
    def Name(self): return self.CONST_TYPE
    def Type(self):
        if self.CONST_NAME:
            return self.CONST_NAME
        else:
            return self.INDEX_NAME
    def Description(self): 
        s = '(' + join(self.FIELD_NAME, ',') + ')'

        if self.CONST_TYPE == 'FOREIGN KEY':
            (index_name, tab, cols) = self.FOREIGN_KEY
            s += ' REFERENCES ' + tab + '(' + join(cols, ',') + ')'
        if self.UPDATE_RULE and self.UPDATE_RULE=='CASCADE':
            s += ' ON UPDATE ' + self.UPDATE_RULE
        if self.DELETE_RULE and self.DELETE_RULE=='CASCADE':
            s += ' ON DELETE ' + self.DELETE_RULE
        return s

class CheckConstraintBrowser(Browser):
    icon='check'+'.gif'
    def tpValues(self):
        return [SourceBrowser(self.CHECK_SOURCE)]

    def tpId(self): 
        return self.CHECK_NAME
    def Name(self): return 'CHECK'
    def Type(self): return self.CHECK_NAME

class TriggerBrowser(Browser):
    icon='trigger'+'.gif'
    def _trigger_prefix(self):
        return (self.TRIGGER_TYPE + 1) & 1
    def _trigger_action_suffix(self, slot):
        suffix_types = ['', 'INSERT', 'UPDATE', 'DELETE']
        i = ((self.TRIGGER_TYPE + 1) >> (slot * 2 -1)) & 3
        return suffix_types[i]

    def tpValues(self):
        source = self._trigger_action_suffix(1)

        s = self._trigger_action_suffix(2)
        if s:
            if source: source += 'OR '
            source += s

        s = self._trigger_action_suffix(3)
        if s:
            if source: source += 'OR '
            source += s


        if self._trigger_prefix() == 0:
            source = 'BEFORE ' + source
        else:
            source = 'AFTER ' + source
        if self.TRIGGER_SOURCE:
            source += ' ' + self.TRIGGER_SOURCE
        else:
            source += ' [------ No soruce ------]' # System table

        return [SourceBrowser(source)]

    def tpId(self): 
        return self.TRIGGER_NAME
    def Type(self): return 'TRIGGER'
    def Description(self):
        return '[' + str(self.TRIGGER_SEQUENCE) + ']'

class SourceBrowser(Base, Acquisition.Implicit):
    icon=''
    Description=''

    def __init__(self, source, params=None):
        self._source=source
        self._params=params

    def tpValues(self):
        return []

    def tpId(self): return ''
    def Name(self): return ''
    def Type(self): return ''
    def Source(self): return self._source
    def Params(self): return self._params




##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################
import os, thread, string
import firebirdsql
import Shared.DC.ZRDB.THUNK
from DateTime import DateTime

from Products.FirebirdDA import QueryError

with_system_flag=0

def fetchallmap(c):
    desc = c.description
    r = []
    for row in c.fetchall():
        d = {}
        for i in range(len(desc)):
            key = str(desc[i][0])
            try: d[key] = string.strip(row[i])
            except: d[key] = row[i]
        r.append(d)
    return r

class DB(Shared.DC.ZRDB.THUNK.THUNKED_TM):
    database_error=firebirdsql.Error
    opened=None

    def tables(self, *args, **kw):
        c = self.conn.cursor()

        if with_system_flag:
            c.execute('''select rdb$relation_name TABLE_NAME, 
                rdb$owner_name TABLE_OWNER, rdb$view_source VIEW_SOURCE,
                rdb$system_flag SYSTEM_FLAG
                from rdb$relations 
                order by rdb$flags, rdb$relation_name''')
        else:
            c.execute('''select rdb$relation_name TABLE_NAME, 
                rdb$owner_name TABLE_OWNER, rdb$view_source VIEW_SOURCE,
                rdb$system_flag SYSTEM_FLAG
                from rdb$relations where rdb$flags=1 
                order by rdb$relation_name''')
        rows = fetchallmap(c)
        return rows

    def columns(self, table_name):
        c = self.conn.cursor()

        r=c.execute('''select A.rdb$field_name NAME, 
    A.rdb$null_flag NULL_FLAG, A.rdb$default_source DEFAULT_SOURCE,
    C.rdb$type_name TYPE_NAME,
    B.rdb$field_sub_type FIELD_SUB_TYPE, B.rdb$field_precision FIELD_PRECISION,
    B.rdb$field_scale FIELD_SCALE, B.rdb$character_length "CHARACTER_LENGTH"
    from rdb$relation_fields A, rdb$fields B, rdb$types C
    where C.rdb$field_name='RDB$FIELD_TYPE' 
        and A.rdb$field_source = B.rdb$field_name 
        and B.rdb$field_type=C.rdb$type and  A.rdb$relation_name='%s'
    order by A.rdb$field_position, A.rdb$field_name''' % table_name)

        rows=fetchallmap(c)

        return rows

    def constraints(self, table_name):
        c = self.conn.cursor()

        r=c.execute('''select 
    a.rdb$index_name INDEX_NAME, a.rdb$index_id INDEX_ID, 
    a.rdb$foreign_key FOREIGN_KEY, 
    b.rdb$field_name FIELD_NAME, c.rdb$constraint_type CONST_TYPE, 
    c.rdb$constraint_name CONST_NAME,
    d.rdb$update_rule UPDATE_RULE, d.rdb$delete_rule DELETE_RULE
    from rdb$indices A, rdb$index_segments B, 
    rdb$relation_constraints C left join rdb$ref_constraints D 
                on c.rdb$constraint_name=d.rdb$constraint_name
    where A.rdb$index_name=B.rdb$index_name 
        AND A.rdb$index_name=C.rdb$index_name
        AND a.rdb$relation_name='%s'
    order by a.rdb$index_id, b.rdb$field_position ''' % table_name)
        rows = fetchallmap(c)

        d = {}
        for row in rows:
            if not d.has_key(row['INDEX_ID']):
                if row['CONST_TYPE']:
                    const_type = row['CONST_TYPE']
                else:
                    const_type = 'INDEX'
                d[row['INDEX_ID']] = {
                    'INDEX_NAME': row['INDEX_NAME'], 
                    'CONST_TYPE': const_type, 
                    'CONST_NAME': row['CONST_NAME'],
                    'UPDATE_RULE': row['UPDATE_RULE'],
                    'DELETE_RULE': row['DELETE_RULE'],
                    'FIELD_NAME': [] }
                if row['FOREIGN_KEY']:
                    d[row['INDEX_ID']]['FOREIGN_KEY'] = self._references(c, row['FOREIGN_KEY'])

            d[row['INDEX_ID']]['FIELD_NAME'].append(row['FIELD_NAME'])


        return d

    def _references(self, c, index_name):
        r=c.execute('''select 
            a.rdb$relation_name RELATION_NAME, B.rdb$field_name FIELD_NAME 
              from rdb$indices A, rdb$index_segments B
              where A.rdb$index_name='%s' 
              and A.rdb$index_name=b.rdb$index_name''' % index_name)

        rows = c.fetchall()
        d = []
        for r in rows:
            d.append(string.strip(r[1]))

        return (index_name, string.strip(rows[0][0]), d)  #index,table,[fields]

    def triggers(self, table_name):
        c = self.conn.cursor()

        if with_system_flag:
            r=c.execute(''' select 
                rdb$trigger_name TRIGGER_NAME, 
                rdb$trigger_sequence TRIGGER_SEQUENCE, 
                rdb$trigger_type TRIGGER_TYPE, 
                rdb$trigger_source TRIGGER_SOURCE, 
                rdb$trigger_inactive TRIGGER_INACTIVE
                    from rdb$triggers
                    where rdb$relation_name='%s'
                    order by rdb$trigger_sequence''' % table_name)
        else:
            r=c.execute(''' select 
                rdb$trigger_name TRIGGER_NAME, 
                rdb$trigger_sequence TRIGGER_SEQUENCE, 
                rdb$trigger_type TRIGGER_TYPE, 
                rdb$trigger_source TRIGGER_SOURCE, 
                rdb$trigger_inactive TRIGGER_INACTIVE
                    from rdb$triggers
                    where (rdb$system_flag is null or rdb$system_flag = 0) 
                        and rdb$relation_name='%s'
                    order by rdb$trigger_sequence''' % table_name)

        rows = fetchallmap(c)
        return rows

    def check_constraints(self, table_name):
        c = self.conn.cursor()

        r=c.execute('''select 
            A.rdb$constraint_name CHECK_NAME, 
            C.rdb$trigger_source CHECK_SOURCE 
            from rdb$relation_constraints A, rdb$check_constraints B, 
                rdb$triggers C
            where A.rdb$constraint_name = B.rdb$constraint_name 
                and B.rdb$trigger_name = C.rdb$trigger_name 
                and rdb$constraint_type='CHECK' 
                and A.rdb$relation_name = '%s' ''' % table_name)

        rows = c.fetchall()

        d = {}
        for row in rows:
            d[row[0]] = {'CHECK_NAME': string.strip(row[0]),
                        'CHECK_SOURCE': string.strip(row[1])}


        return d

    def generators(self):
        c = self.conn.cursor()

        if with_system_flag:
            c.execute('''select rdb$generator_name, rdb$system_flag
                  from rdb$generators 
                  order by rdb$system_flag, rdb$generator_name''')
        else:
            c.execute('''select rdb$generator_name, rdb$system_flag
                  from rdb$generators 
                  where rdb$system_flag is null or rdb$system_flag = 0 
                  order by rdb$system_flag, rdb$generator_name''')
        gens=c.fetchall()
        r=[]
        for gen in gens:
            c.execute(''' select gen_id(%s,0) from rdb$database ''' % gen[0])
            count = c.fetchone()[0]
            r.append({'GENERATOR_NAME': string.strip(gen[0]),
                    'SYSTEM_FLAG': gen[1],
                    'GENERATOR_COUNT': count})

        return r

    def procedures(self):
        c = self.conn.cursor()

        c.execute('''select rdb$procedure_name, rdb$procedure_source 
                from rdb$procedures order by rdb$procedure_name''')
        procs=c.fetchall()
        r=[]
        for proc in procs:
            c.execute(''' select A.rdb$parameter_name NAME, 
    C.rdb$type_name TYPE_NAME,
    B.rdb$field_sub_type FIELD_SUB_TYPE, B.rdb$field_precision FIELD_PRECISION,
    B.rdb$field_scale FIELD_SCALE, B.rdb$character_length "CHARACTER_LENGTH"
    from rdb$procedure_parameters A, rdb$fields B, rdb$types C
      where C.rdb$field_name='RDB$FIELD_TYPE' 
         and A.rdb$field_source = B.rdb$field_name 
         and A.rdb$parameter_type = 0
         and B.rdb$field_type=C.rdb$type and  A.rdb$procedure_name='%s'
       order by A.rdb$parameter_number''' % proc[0])
            in_params = fetchallmap(c)

            c.execute(''' select A.rdb$parameter_name NAME, 
    C.rdb$type_name TYPE_NAME,
    B.rdb$field_sub_type FIELD_SUB_TYPE, B.rdb$field_precision FIELD_PRECISION,
    B.rdb$field_scale FIELD_SCALE, B.rdb$character_length "CHARACTER_LENGTH"
    from rdb$procedure_parameters A, rdb$fields B, rdb$types C
      where C.rdb$field_name='RDB$FIELD_TYPE' 
         and A.rdb$field_source = B.rdb$field_name 
         and A.rdb$parameter_type = 1
         and B.rdb$field_type=C.rdb$type and  A.rdb$procedure_name='%s'
       order by A.rdb$parameter_number''' % proc[0])
            out_params = fetchallmap(c)

            r.append({'PROCEDURE_NAME': proc[0],
                    'PROCEDURE_SOURCE': proc[1],
                    'IN_PARAMS': in_params,
                    'OUT_PARAMS':out_params})

        return r

    def open(self):
        self.conn = firebirdsql.connect(**self.conn_args)
        self.conn.set_autocommit(True)
        self.opened=DateTime()

    def close(self):
        self.conn.close()
        self.opened = None

    def __init__(self,connection):
        conn_args = {}
        for s in string.split(connection):
            k,v = s.split('=', 1)
            conn_args[k] = v
        self.conn_args = conn_args
        self.db_conn = []
        self.open()

    def query(self,query_string, max_rows=9999999):
        c = self.conn.cursor()

        queries=filter(None, map(string.strip,string.split(query_string, '\0')))
        if not queries: raise QueryError, 'empty query'
        desc=None
        result=[]
        for qs in queries:
            c.execute(qs)
            d=c.description
            if d is None: continue
            if desc is None: desc=d
            elif d != desc:
                raise QueryError, (
                    'Multiple incompatible selects in '
                    'multiple sql-statement query'
                    )

            if len(result) < max_rows:
                r = c.fetchmany(max_rows-len(result))
                if r:
                    result += r


        if desc is None:
            return (), ()

        items=[]
        for name, type, width, ds, p, scale, null_ok in desc:
            if type=='NUMBER':
                if scale==0: type='i'
                else: type='n'
            elif type=='DATE':
                type='d'
            else: type='s'
            items.append({
                'name': name,
                'type': type,
                'width': width,
                'null': null_ok,
                })

        return items, result




##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################
from App.ImageFile import ImageFile
import os

classes=('DA.Connection',)
database_type='Firebird'

class FirebirdError(Exception):
    pass

class QueryError(FirebirdError):
    pass

misc_={}

for icon in ('conn', 'table', 'view', 'stable', 'what',
             'field', 'text','bin','int','float',
             'date','time','datetime', 'generator', 'sequence',
             'procedure', 'trigger', 'trigger_inactive', 
             'key_primary', 'key_foreign', 'key', 'index', 'check'):
    misc_[icon+'.gif']=ImageFile('icons/%s.gif' % icon, globals())

DA=None
def getDA():
    global DA
    if DA is None:
        import DA
    return DA

getDA()

__module_aliases__=(
    ('Products.AqueductFirebird.DA', DA),
    )

def manage_addFirebirdConnectionForm(self, REQUEST, *args, **kw):
    " "
    DA=getDA()
    return DA.addConnectionForm(
        self, self, REQUEST,
        database_type=database_type)

def manage_addFirebirdConnection(
    self, id, title, connection, check=None, REQUEST=None):
    " "
    return getDA().manage_addFirebirdConnection(
        self, id, title, connection, check, REQUEST)

def initialize(context):

    context.registerClass(
        DA.Connection,
        permission='Add Firebird Database Connections',
        constructors=(manage_addFirebirdConnectionForm,
                      manage_addFirebirdConnection),
        legacy=(manage_addFirebirdConnectionForm,
                manage_addFirebirdConnection),
    )




##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################
database_type='Firebird'

from db import DB
import sys, DABase
import Shared.DC.ZRDB.Connection, ThreadLock
from App.special_dtml import HTMLFile
from zExceptions import BadRequest

_Connection=Shared.DC.ZRDB.Connection.Connection

_connections={}
_connections_lock=ThreadLock.allocate_lock()

addConnectionForm=HTMLFile('dtml/connectionAdd',globals())
def manage_addFirebirdConnection(
    self, id, title, connection, check=None, REQUEST=None):
    """Add a DB connection to a folder"""

    # Note - type checking is taken care of by _setObject
    # and the Connection object constructor.
    self._setObject(id, Connection(
        id, title, connection, check))
    if REQUEST is not None: return self.manage_main(self,REQUEST)

class Connection(DABase.Connection):
    " "
    database_type=database_type
    id='%s_database_connection' % database_type
    meta_type=title='%s Database Connection' % database_type
    icon='misc_/%sDA/conn.gif' % database_type

    manage_properties=HTMLFile('dtml/connectionEdit', globals())

    def connected(self):
        if hasattr(self, '_v_database_connection'):
            return self._v_database_connection.opened
        return ''

    def title_and_id(self):
        s=_Connection.inheritedAttribute('title_and_id')(self)
        if (hasattr(self, '_v_database_connection') and
            self._v_database_connection.opened):
            s="%s, which is connected" % s
        else:
            s="%s, which is <font color=red> not connected</font>" % s
        return s

    def title_or_id(self):
        s=_Connection.inheritedAttribute('title_and_id')(self)
        if (hasattr(self, '_v_database_connection') and
            self._v_database_connection.opened):
            s="%s (connected)" % s
        else:
            s="%s (<font color=red> not connected</font>)" % s
        return s

    def connect(self,s):
        _connections_lock.acquire()
        try:
            c=_connections
            self._v_database_connection=c[s]=DB(s)
            return self
        finally:
            _connections_lock.release()



from setuptools import setup, find_packages

version = '0.6.4'

setup(name='Products.FirebirdDA',
      version=version,
      description="Firebird database adapter for Zope2",
      long_description=open("README.rst").read(),
      # Get more strings from
      # http://pypi.python.org/pypi?:action=list_classifiers
      classifiers=[
        "Programming Language :: Python",
        ],
      keywords='Firebird',
      author='Hajime Nakagami',
      author_email='nakagami@gmail.com',
      url='https://github.com/nakagami/Products.FirebirdDA',
      license='ZPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['Products'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'firebirdsql',
          'ThreadLock',
          'Products.ZSQLMethods',
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



import unittest

from zope.testing import doctestunit
from zope.component import testing
from Testing import ZopeTestCase as ztc

def test_suite():
    return unittest.TestSuite([

        # Unit tests for your API
        doctestunit.DocFileSuite(
            'README.txt', package='Products.FirebirdDA',
            setUp=testing.setUp, tearDown=testing.tearDown),

        #doctestunit.DocTestSuite(
        #    module='Products.FirebirdDA.mymodule',
        #    setUp=testing.setUp, tearDown=testing.tearDown),

        # Integration tests that use ZopeTestCase
        #ztc.ZopeDocFileSuite(
        #    'README.txt', package='Products.FirebirdDA',
        #    setUp=testing.setUp, tearDown=testing.tearDown),

        #ztc.FunctionalDocFileSuite(
        #    'browser.txt', package='Products.FirebirdDA'),

        ])

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')
